<?php

	// $output_string_ - a reference to the string to which the output is written
	// $conn           - connection to the database
	// returns the number of selected nutrients
	function getNutrientsInJSON(&$output_string_, $conn, $query_conditions_){
		// the number of selected nutrients
		$selected_count = 0;

		// a one dimensional array of the selected nutrients from the saved query
		$selection = array();

		// getting the seleted nutrients and feeds from the saved query, or the selected nutrients from the query result
		if (isset($query_conditions_->sel_nutrients)&& !($query_conditions_->sel_nutrients === "")){
			$sel = explode(",", $query_conditions_->sel_nutrients);
			for($i =0; $i < count($sel); $i++) {
				$selection[$sel[$i]] = $i;
			}		
		}
		if (isset($_SESSION['user'])&&isset($_SESSION['userlevel'])&&$_SESSION['userlevel'] >= 2){ 
			$selected_classified_feeds   = $query_conditions_->sel_classified_feeds;
		}
		if (isset($_SESSION['user'])&&isset($_SESSION['userlevel'])&&$_SESSION['userlevel'] > 2){
			$selected_unclassified_feeds = $query_conditions_->sel_unclassified_feeds;
		}
		$selected_agridea_feeds      = $query_conditions_->sel_agridea_feeds;		

		// selecting and preparing the result for jstree, i.e., as xml nodes
		// we use a data cube for the fast retrieval
		$select_nutrients = "select d_nutrient.abbreviation_de as name_de, ".
				"array_to_string(array_agg(distinct d_nutrient.description_de), ',') as description_de, ".
				"array_to_string(array_agg(distinct d_nutrient.nutrient_key), ',') as id, ".
				"array_to_string(array_agg(distinct d_nutrient.abbreviation_".$_SESSION['lng']."), ',') as name, ".
				"array_to_string(array_agg(distinct d_nutrient.description_".$_SESSION['lng']."), ',') as description ".
				"from d_nutrient where d_nutrient.nutrient_key in (select distinct nutrient_key from ";
				if ($_SESSION["data_source"] === "fact_table") $select_nutrients .= "vr_cube where ";
				else $select_nutrients .= "vc_cube where ";

		$search_parameters = FALSE;

		if(isset($selected_unclassified_feeds)&&strlen($selected_unclassified_feeds)>0){
			if ($search_parameters) $select_nutrients .= " or feed_key in ($selected_unclassified_feeds)";
			else {
				$select_nutrients .= "feed_key in ($selected_unclassified_feeds)";
				$search_parameters = TRUE;
			}
		}

		if(isset($selected_agridea_feeds)&&strlen($selected_agridea_feeds)>0){
			if ($search_parameters) $select_nutrients .= " or feed_key in ($selected_agridea_feeds)";
			else {
				$select_nutrients .= "feed_key in ($selected_agridea_feeds)";
				$search_parameters = TRUE;
			}	
		}

		if(isset($selected_classified_feeds)&&strlen($selected_classified_feeds)>0){
			if ($search_parameters) $select_nutrients .= " or feed_key in ($selected_classified_feeds)";
			else {
				$select_nutrients .= "feed_key in ($selected_classified_feeds)";
				$search_parameters = TRUE;
			}		
		}

		$select_nutrients .= ") group by d_nutrient.abbreviation_de order by name_de";

		$result = pg_query($conn, $select_nutrients);
		if (!$result) die('Invalid query: ' . pg_error());

		$rows   = pg_numrows($result);
		
		$output_string_.= '{"c":'.$rows.',';
		$html_val = "";
		for ($i = 0; $i < $rows; $i++){
			$row            = pg_fetch_array($result, $i);
			$id             = $row['id'];
			$description    = $row['description'];
			$name           = $row['name'];
			$name_de        = $row['name_de'];
			$description_de = $row['description_de'];
			$sel            = false;
			$sel1           = explode(",", $id);
			for($ii =0; $ii < count($sel1) && !($sel); $ii++) {
				if (isset($selection[$sel1[$ii]])){
					$sel = true;
					$selected_count++;
				}
			}

			if (isset($name)) $name = htmlspecialchars($name, ENT_COMPAT, 'UTF-8');
			else{
				if (isset($name_de)){
					$name = htmlspecialchars($name_de, ENT_COMPAT, 'UTF-8');
					$description = $description_de;
				}else{
					$name = "n/a";
				}
			}

			if(isset($description)){
				$description = htmlspecialchars($description, ENT_COMPAT, 'UTF-8');
				if ($sel) $html_val.="<li id='$id' class=\"jstree-checked\" title='$name'><a href='#'>$name   <i>-- $description</i></a></li>";
				else $html_val.="<li id='$id' title='$name'><a href='#'>$name   <i>-- $description</i></a></li>";
			}else {
				if ($sel) $html_val.="<li id='$id' class=\"jstree-checked\" title='$name'><a href='#'>$name</a></li>";
				else $html_val.="<li id='$id' title='$name'><a href='#'>$name</a></li>";
			}
		}

		unset($selection);
		$output_string_.='"v":';
		$output_string_.=json_encode($html_val);
		$output_string_.=',"s":'.$selected_count.'}';
		
		return $selected_count;
	}


/*	for testing individually:
	ob_start();
	require './db_info.php';
    	require './session_info.php';
	ob_end_clean();

	// connecting to the database
	$conn = pg_connect("host=$host port=$port dbname=$database user=$username password = $password") or die  ('Error connecting to Postgres');
	if (!$conn) die('Not connected : ' . pg_error());

	$output_string = "";
	getNutrientsInJSON($output_string, $conn);
	echo '(';
	echo $output_string;
	echo ')';

	pg_close($conn);
	exit;
*/
?>
