<?php
	require './db_info.php';
	require './ajax-pg-query-where.php';
	
	$where = getWhereStatement();

	// creating queries
	$query_one = "select distinct lims_number, postal_code, city, canton, latitude, longitude ".
			"from d_feed, fact_table, d_nutrient, d_origin, d_time ".
			"where fact_table.id_feed_fkey = d_feed.feed_key ".
			"and fact_table.id_nutrient_fkey=d_nutrient.nutrient_key ".
			"and fact_table.id_origin_fkey = d_origin.origin_key ".
			"and fact_table.id_time_fkey = d_time.time_key and d_m_b<> true $where".
			" order by canton, postal_code, lims_number";
	$query_two = "select distinct lims_number, z_abbreviation_de, avg(quantity) as avg_quantity ".
			"from d_feed, fact_table, d_nutrient, d_origin, d_time ".
			"where fact_table.id_feed_fkey = d_feed.feed_key ".
			"and fact_table.id_nutrient_fkey=d_nutrient.nutrient_key ".
			"and fact_table.id_origin_fkey = d_origin.origin_key ".
			"and fact_table.id_time_fkey = d_time.time_key and d_m_b<> true $where".
			" group by lims_number, z_abbreviation_de order by lims_number";
	
	define("MAPS_HOST", "maps.google.ch");
	define("MAPS_STATUS_OK",               "OK");
	define("MAPS_STATUS_ZERO",             "ZERO_RESULTS");
	define("MAPS_STATUS_OVER_QUERY_LIMIT", "OVER_QUERY_LIMIT");
	define("MAPS_STATUS_REQUEST_DENIED",   "REQUEST_DENIED");
	define("MAPS_STATUS_INVALID_REQUEST",  "INVALID_REQUEST");

	function parseToXML($htmlStr){
		$xmlStr=str_replace('<','&lt;',$htmlStr);
		$xmlStr=str_replace('>','&gt;',$xmlStr);
		$xmlStr=str_replace('"','&quot;',$xmlStr);
		$xmlStr=str_replace("'",'&#39;',$xmlStr);
		$xmlStr=str_replace("&",'&amp;',$xmlStr);
		return $xmlStr;
	}

	// Start XML file, create parent node
	$dom = new DOMDocument("1.0", "UTF-8");
	$node = $dom->createElement("markers");
	$parnode = $dom->appendChild($node);

	//  Connection to PostgreSQL-DB
	$conn = pg_connect("host=$host port=$port dbname=$database user=$username password = $password") or die  ('Error connecting to Postgres');
	if (!$conn) die('Not connected : ' . pg_error());

	$result = pg_query($conn, $query_one);
	$numrows = pg_numrows($result);

	if (!$result) {
		echo "Error: query wasn't successful (no result)";
		die();
	}


	// Initialize URL and delay in geocode speed
	// Example of possible URL-Request: http://maps.google.com/maps/api/geocode/json?address=1600+Amphitheatre+Parkway,+Mountain+View,+CA&sensor=true_or_false;
	$delay = 0;
	$base_url = "http://" . MAPS_HOST . "/maps/api/geocode/xml?sensor=true";

	header('Content-type: text/xml');

	// Iterate through the rows, geocoding each address and update values in DB
	for($ri = 0; $ri < $numrows; $ri++) {
		$row = pg_fetch_array($result, $ri);

		$latitude = $row['latitude'];
		$longitude = $row['longitude'];

		// add to xml: node for marker with attributes
		$node = $dom->createElement('marker');
		$newnode = $parnode->appendChild($node);
		$newnode->setAttribute("name",$row['lims_number']);
		$newnode->setAttribute("plz", htmlspecialchars($row['postal_code'], ENT_COMPAT, 'UTF-8'));
		$newnode->setAttribute("city", htmlspecialchars($row['city'], ENT_COMPAT, 'UTF-8'));
		$newnode->setAttribute("canton", htmlspecialchars($row['canton'], ENT_COMPAT, 'UTF-8'));
		$newnode->setIdAttribute("name", true);

		if(($latitude != null) && ($longitude != null)){
			$newnode->setAttribute("lat", $latitude);
			$newnode->setAttribute("lng", $longitude);

		}else{

			$geocode_pending = true;

			while ($geocode_pending) {
				$plz = $row["postal_code"];
				$city = $row["city"];
				$canton = $row["canton"];
				$address = $plz + " " + $city + ", " + $canton + ", Switzerland";

				$request_url = $base_url . "&address=" . urlencode($address);
				$xml = simplexml_load_file($request_url) or die("url not loading");

				$status = $xml->status;

				if (strcmp($status, MAPS_STATUS_OK) == 0) {
					// Successful geocode
					$geocode_pending = false;
					$delay = 0;
					$coordinates = $xml->result->geometry->location;

					// result format: Longitude, Latitude, (Altitude)
					$lat = $coordinates->lat;
					$lng = $coordinates->lng;

					$newnode->setAttribute("lat", $lat);
					$newnode->setAttribute("lng", $lng);

					//$lat & $lng can be stored as float(10,6) aka numeric(10,6) with no conversion:
					$update = "UPDATE d_origin SET latitude= ".$lat.", longitude= ".$lng." WHERE postal_code= ".$address;
					$update_result = pg_query($conn, $update);

					if (!$update_result) {
						//...ceep on running, update it next time.
					}


				} else if (strcmp($status, MAPS_STATUS_OVER_QUERY_LIMIT) == 0) {
					// sent geocodes too fast
					$delay += 100000;
				} else {
					// failure to geocode
					$geocode_pending = false;
					$newnode->setAttribute("lat", "error");
					$newnode->setAttribute("error", $status);
				}
					usleep($delay);
			}
		}
	}

	$result2 = pg_query($conn, $query_two);
	$numrows2 = pg_numrows($result2);

	if (!$result2) {
		echo "Error: query wasn't successful (no result)";
		die();
	}else{
		for($rj = 0; $rj < $numrows2; $rj++){
			$row = pg_fetch_array($result2, $rj);
			$nodeToAdd = $dom->getElementById($row['lims_number']);
			if($nodeToAdd != null){
				$node_two = $dom->createElement('nutrient');
				$nutrient = $nodeToAdd->appendChild($node_two);
				$nutrient->setAttribute("abbreviation", $row['z_abbreviation_de']);
				$nutrient->setAttribute("quantity", number_format(floatval($row['avg_quantity']),3));
			}
		}
	}

	pg_close($conn);

	echo $dom->saveXML();
?>
