/**
 * Cumulative F-Distribution
 * 
 * Code for calculating the level of significance to the given F-Value, nominator and denominator.
 * Code was copied from http://eswf.uni-koeln.de/glossar/surfstat/fvert.htm 29.03.2012
 * 
 * Hans-Jrgen Andre 10. September 2001: bernommen und angepat von Gnther Gediga's WWW-Library (Universitt Osnabrck).
 */


var SQRTI2PI = 0.3989422804014327; // = 1/sqrt(2*PI)
HL2PI = 0.91893853320467275; // = log(2PI)/2
UNENDLICH = 1000000000000000000000;
ARGMIN = 6;
SMALL = 0.00000000001;
S2 = 0.1;
EXPMIN = -30.0;
STARTFRACTION = 20;
ITERMAX = 100; // Maximale Iterationsanzahl bei Intervallsplitt
EPSILON = 0.0001; // Epsilon fuer die Genauigkeit invertiert X2 und F-Werte

function pmin(p) {
	if (eval(p) > 0.00001)
		return p;
	if (eval(p) < 1.0e-12)
		return 0;
	if (eval(p) < 1.0e-11)
		return 1.0e-11;
	if (eval(p) < 1.0e-10)
		return 1.0e-10;
	if (eval(p) < 1.0e-9)
		return 1.0e-9;
	if (eval(p) < 1.0e-8)
		return 1.0e-8;
	if (eval(p) < 1.0e-7)
		return 1.0e-7;
	if (eval(p) < 1.0e-6)
		return 1.0e-6;
	return p;
}

function ftail(par1, par2, par3) // Berechnet zu F und dfn(ominator) und
// dfd(enominator) den "Schwanz" der F-Verteilung
{
	var p = 1, retval = 1, F, dfn, dfd;

	F = eval(par1);
	dfn = eval(par2);
	dfd = eval(par3);

	if (F < 0.00001 || dfn <= 0.0 || dfd <= 0.0)
		;
	else if (F * dfn >= dfd || F > 1.0 + 20.0 / dfn + 10.0 / Math.sqrt(dfn))
		p = series(F, dfn, dfd);
	else
		p = 1.0 - series(1.0 / F, dfd, dfn);
	retval = p;
	if (p >= 1.0 || p <= 0.0) {
		if (F > 1.0)
			retval = 0.0;
		else if (F == 1.0)
			retval = 0.5;
		else
			retval = 1.0;
	}
	return retval;
}

function series(par1, par2, par3) // Berechnet zu F und dfn(ominator) und
// dfd(enominator) die unvollstndige Gamma-Funktion.
{
	var x, c, er, s, n, t1, t, F, dfn, dfd, retval;

	F = eval(par1);
	dfn = eval(par2);
	dfd = eval(par3);

	dfn = dfn / 2;
	dfd = dfd / 2;
	x = dfd / (dfd + dfn * F);
	c = logamma(dfn + dfd) - logamma(dfn) - logamma(dfd + 1) + dfd
			* Math.log(x) + dfn * Math.log(1 - x);
	if (c < EXPMIN) {
		return -1.0;
	} else {
		dfn += dfd;
		dfd += 1;
		c = Math.exp(c);
		er = SMALL / c;
		t = dfn * x / dfd;
		t1 = 0;
		s = t + 1;
		n = 0;
		while (t > er || t > t1) {
			n += 1;
			t1 = t;
			t *= ((dfn + n) * x / (dfd + n));
			s += t;
		}
		retval = s * c;
	}
	return retval;
}

function logamma(z) // Berechnet den Logarithmus der Gamma-Funktion
{
	var a, g, i, n;

	a = eval(z);

	n = Math.floor(ARGMIN - a + 0.0001);
	if (n > 0)
		a = a + n;
	g = 1.0 / (a * a);
	g = (1.0 - g
			* (1.0 / 30.0 - g * (1.0 / 105.0 - g * (1.0 / 140.0 - g / 99.0))))
			/ (12.0 * a);
	g = g + ((a - 0.5) * Math.log(a) - a + HL2PI);
	for (i = 0; i < n; i++) {
		a = a - 1.0;
		g = g - Math.log(a);
	}
	return g;
}

function RoundDP(x, dp) {
	// Rounds x to dp decimal places.
	var powten = Math.pow(10, dp);
	return (Math.round(x * powten) / powten)
}