function DensityOverlay(map, _copy_of_query_parameters){
    this.canvas = null;
    this.div = null;
    this.bounds = null;
    this.divBounds = null;
    this.setMap(map);
    this.scale = 10;
    this.opacity = 180;
    this._copy_of_query_parameters = _copy_of_query_parameters;
    this.is_visible = false;
}

DensityOverlay.prototype = new google.maps.OverlayView();

DensityOverlay.prototype.onRemove = function(){
    delete this.canvas;
    this.canvas = null;
    this.div.parentNode.removeChild(this.div);
    this.div = null;
}

DensityOverlay.prototype.draw = function(){
    if (!this.div || !this.canvas) return;

    var currentBounds = this.map.getBounds();    
    if (currentBounds.equals(this.bounds)) return;
    
    this.bounds = currentBounds;    
    this.scaleDiv();
}

DensityOverlay.prototype.scaleDiv = function(){
	var overlayProjection = this.getProjection();
	var sw = overlayProjection.fromLatLngToDivPixel(this.divBounds.getSouthWest());
	var ne = overlayProjection.fromLatLngToDivPixel(this.divBounds.getNorthEast());

	var div = this.div;
	div.style.left = sw.x + 'px';
	div.style.top = ne.y + 'px';
	div.style.width = (ne.x - sw.x) + 'px';
	div.style.height = (sw.y - ne.y) + 'px';
}

function red(density){
	if (density <= 0.5) return 0;
	if (density >= 0.75) return 255;
	return Math.round((density-0.5)/0.25*255);
}

function green(density){
	if (density >= 0.25 && density <= 0.75) return 255;
	if (density < 0.25) return Math.round(density/0.25*255);
	return Math.round((1-((density-0.75)/0.25))*255);
}

function blue(density){
	if (density <= 0.25) return 255;
	if (density >= 0.5) return 0;
	return Math.round((1-((density-0.25)/0.25))*255);
}

DensityOverlay.prototype.setData = function (divBounds, width, height, max, values,n){
	var middle = new Date();
	this.divBounds = divBounds;	
	
    	var canvas_colors = document.createElement('canvas');
	canvas_colors.width = width;
	canvas_colors.height = height;

	var context_colors = canvas_colors.getContext( '2d' );
	context_colors.fillStyle = 'rgba(1,0,0,1)';
	context_colors.fillRect( 0, 0, width, height );

	var image_colors = context_colors.getImageData( 0, 0, width, height );
	var data = image_colors.data;
	
	for (var i = 0; i < values.length; i++){
		var density = values[i] / max;		
		data[i*4] = red(density);
		data[i*4+1] = green(density);
		data[i*4+2] = blue(density);
		data[i*4+3] = (density==0) ? 0 : 180;
	}
	
	context_colors.putImageData( image_colors, 0, 0 );	
	
	this.div = document.createElement('DIV');
	this.div.style.borderStyle = "none";
	this.div.style.borderWidth = "0px";
	this.div.style.position = "absolute";
	
	this.canvas = document.createElement("canvas");
	this.canvas.style.width = "100%";
	this.canvas.style.height = "100%";
	this.canvas.width = this.scale*width;
	this.canvas.height = this.scale*height;
	if (!this.is_visible) this.div.style.visibility = "hidden";
	else this.div.style.visibility = "visible";
	
	var context_canvas = this.canvas.getContext( '2d' );
	context_canvas.scale( this.scale, this.scale );
	context_canvas.drawImage( canvas_colors, 0,0);
		
	this.div.appendChild(this.canvas);
	
	var panes = this.getPanes();
	panes.overlayLayer.appendChild(this.div);

	document.getElementById("_maps_div_top_label").innerHTML = Math.round(max);
	document.getElementById("_maps_div_bottom_label").innerHTML = 0;
	
	this.scaleDiv();
}

DensityOverlay.prototype.calculateImage = function(){
	this.start = new Date();

	var httpObject = createHttpObjectFromParameters("db_scripts/ajax-pg-result-density.php",this._copy_of_query_parameters);
	if (httpObject != null){
		httpObject.onreadystatechange = function(){
			if(httpObject.readyState == 4){
				var resultXML = httpObject.responseXML.documentElement;
				var resultArray = resultXML.getElementsByTagName("data");
				
				var swBound = new google.maps.LatLng(resultXML.getElementsByTagName("sw")[0].getAttribute("lat"), resultXML.getElementsByTagName("sw")[0].getAttribute("lng"));
                		var neBound = new google.maps.LatLng(resultXML.getElementsByTagName("ne")[0].getAttribute("lat"), resultXML.getElementsByTagName("ne")[0].getAttribute("lng"));
                		bounds = new google.maps.LatLngBounds(swBound, neBound);

		                var width = resultXML.getElementsByTagName("grid")[0].getAttribute("width");
                		var height = resultXML.getElementsByTagName("grid")[0].getAttribute("height");
                
		                var max = resultXML.getElementsByTagName("values")[0].getAttribute("max");
                
		                var valuesArray = resultXML.getElementsByTagName("row");
		                var valuesLength = resultXML.getElementsByTagName("row").length;
                		var values = new Array();
                
		                for (var i = 0; i < valuesLength; i++){
		                	var valuesString = valuesArray[i].getAttribute("values");
                			var newValues = valuesString.split(";");
                			values.push.apply(values, newValues);
		                }
                
                		g_densityMap.setData(bounds, width, height, max, values, resultXML.getElementsByTagName("values")[0].getAttribute("n"));
            		}
		}
	}
}

DensityOverlay.prototype.hide = function() {
	if (this.div)
		this.div.style.visibility = "hidden";
	this.is_visible = false;
}

DensityOverlay.prototype.show = function() {
	if (this.div)
		this.div.style.visibility = "visible";
	this.is_visible = true;
}
DensityOverlay.prototype.isCalculated = function() {
	return (this.canvas != null);
}
