function RegressionOverlay(map, _copy_of_query_parameters){
    this.nutrients = new Array();
    this.bounds = null;
    this.setMap(map);
    this.scale = 10;
    this.opacity = 180;
    this._copy_of_query_parameters = _copy_of_query_parameters;
    this.is_visible = false;
}

RegressionOverlay.prototype = new google.maps.OverlayView();

RegressionOverlay.prototype.onRemove = function(){
	for (var n in this.nutrients){
		delete n["canvas"];
		n["div"].parentNode.removeChild(n["div"]);
	}
	this.nutrients = null;
}

RegressionOverlay.prototype.draw = function(){
    var currentBounds = this.map.getBounds();
    
    if (currentBounds.equals(this.bounds)){
	return;
    }
    
    this.bounds = currentBounds;    
    this.scaleDiv();
}

RegressionOverlay.prototype.scaleDiv = function(){
	var overlayProjection = this.getProjection();
	for (var n in this.nutrients){
		var sw = overlayProjection.fromLatLngToDivPixel(this.nutrients[n]["divBounds"].getSouthWest());
		var ne = overlayProjection.fromLatLngToDivPixel(this.nutrients[n]["divBounds"].getNorthEast());
	
		var div = this.nutrients[n]["div"];
		div.style.left = sw.x + 'px';
		div.style.top = ne.y + 'px';
		div.style.width = (ne.x - sw.x) + 'px';
		div.style.height = (sw.y - ne.y) + 'px';
	}
}

function red(density){
	if (density <= 0.5) return 0;
	if (density >= 0.75) return 255;
	return Math.round((density-0.5)/0.25*255);
}

function green(density){
	if (density >= 0.25 && density <= 0.75) return 255;
	if (density < 0.25) return Math.round(density/0.25*255);
	return Math.round((1-((density-0.75)/0.25))*255);
}

function blue(density){
	if (density <= 0.25) return 255;
	if (density >= 0.5) return 0;
	return Math.round((1-((density-0.25)/0.25))*255);
}

RegressionOverlay.prototype.setData = function (divBounds, width, height, max, min, values, nutrient){
	var middle = new Date();
	var opacity = 180;
	
    	var canvas_colors = document.createElement( 'canvas' );
	canvas_colors.width = width;
	canvas_colors.height = height;
	var context_colors = canvas_colors.getContext( '2d' );
	context_colors.fillStyle = 'rgba(0,0,0,1)';
	context_colors.fillRect( 0, 0, width, height );

	var image_colors = context_colors.getImageData( 0, 0, width, height );
	var data = image_colors.data;
	
	for (var i = 0; i < values.length; i++){
		var density = (values[i]-min) / (max-min);
		data[i*4] = red(density);
		data[i*4+1] = green(density);
		data[i*4+2] = blue(density);
		data[i*4+3] = opacity;
		if (values[i] == 0) data[i*4+3] = 0;
		var v = Math.floor((values[i]-min) / (max-min) * 255);
	}
	
	context_colors.putImageData( image_colors, 0, 0 );
	
	var div = document.createElement('DIV');
	div.style.borderStyle = "none";
	div.style.borderWidth = "0px";
	div.style.position = "absolute";
	
	var canvas = document.createElement("canvas");
	canvas.style.width = "100%";
	canvas.style.height = "100%";
	canvas.width = this.scale*width;
	canvas.height = this.scale*height;
	
	var context_canvas = canvas.getContext('2d');
	context_canvas.scale( this.scale, this.scale );
	context_canvas.drawImage( canvas_colors, 0,0);
	
	div.appendChild(canvas);
	
	var panes = this.getPanes();
	panes.overlayLayer.appendChild(div);

	max = Math.round(max*100)/100;
	min = Math.round(min*100)/100;
	
	this.nutrients[nutrient] = { "div": div, "divBounds": divBounds, "canvas": canvas, "max": max, "min": min};	

	document.getElementById("_maps_div_top_label").innerHTML = max;
	document.getElementById("_maps_div_bottom_label").innerHTML = min;
	
	this.scaleDiv();	

	if (this.is_visible) this.nutrients[nutrient]["div"].style.visibility = "visible";
	else this.nutrients[nutrient]["div"].style.visibility = "hidden";
}

RegressionOverlay.prototype.calculateImage = function(nutrient){
	var httpObject = getHttpObject();
	if (httpObject != null){
		var parameters = this._copy_of_query_parameters+"&nutrient=";
		if (nutrient != "-")
			parameters += nutrient;
		httpObject.open("POST", "db_scripts/ajax-pg-result-regression.php", true);
		httpObject.setRequestHeader("Content-Type", "application/x-www-form-urlencoded");
		httpObject.send(parameters);
		httpObject.onreadystatechange = function(){
			if(httpObject.readyState == 4){
				var resultXML = httpObject.responseXML.documentElement;				
				
				var swBound = new google.maps.LatLng(resultXML.getElementsByTagName("sw")[0].getAttribute("lat"), resultXML.getElementsByTagName("sw")[0].getAttribute("lng"));
                		var neBound = new google.maps.LatLng(resultXML.getElementsByTagName("ne")[0].getAttribute("lat"), resultXML.getElementsByTagName("ne")[0].getAttribute("lng"));
		                bounds = new google.maps.LatLngBounds(swBound, neBound);

                		var width = resultXML.getElementsByTagName("grid")[0].getAttribute("width");
		                var height = resultXML.getElementsByTagName("grid")[0].getAttribute("height");

                		var max = resultXML.getElementsByTagName("values")[0].getAttribute("max");
		                var min = resultXML.getElementsByTagName("values")[0].getAttribute("min");
                
                		var valuesArray = resultXML.getElementsByTagName("row");
		                var valuesLength = resultXML.getElementsByTagName("row").length;
		                var values = new Array();
                
                		for (var i = 0; i < valuesLength; i++){
		                	var valuesString = valuesArray[i].getAttribute("values");
                			var newValues = valuesString.split(";");
                			values.push.apply(values, newValues);
                		}
                
                		g_regressionMap.setData(bounds, width, height, max, min, values, nutrient);        	
            		}
		}
	}
}

RegressionOverlay.prototype.hide = function(nutrient) {
	this.is_visible = false;
	for (var n in this.nutrients)
	{
		this.nutrients[n]["div"].style.visibility = "hidden";
	}
}

RegressionOverlay.prototype.show = function(nutrient) {
	for (var n in this.nutrients)
	{
		this.nutrients[n]["div"].style.visibility = "hidden";
	}

	this.is_visible = true;

	if(!this.isCalculated(nutrient))
	{		
		this.calculateImage(nutrient);
	} else {		
		this.nutrients[nutrient]["div"].style.visibility = "visible";
		document.getElementById("_maps_div_top_label").innerHTML = this.nutrients[nutrient]["max"];
		document.getElementById("_maps_div_bottom_label").innerHTML = this.nutrients[nutrient]["min"];
	}	
}
RegressionOverlay.prototype.isCalculated = function(nutrient) {
	return (nutrient in this.nutrients);
}
